/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2020                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#include "blade-config.h"
#include <cmath>

#include "ColorMap.h"
#include  "TraceGlobals.h"

extern  TraceGlobals * trace_globals;


const OTF2_Paradigm   ColorPalette::otf2_paradigms[ NUM_OTF_PARADIGMS ] =   
                                                            {   OTF2_PARADIGM_UNKNOWN, 
                                                                OTF2_PARADIGM_USER, 
                                                                OTF2_PARADIGM_COMPILER, 
                                                                OTF2_PARADIGM_OPENMP, 
                                                                OTF2_PARADIGM_MPI, 
                                                                OTF2_PARADIGM_CUDA, 
                                                                OTF2_PARADIGM_MEASUREMENT_SYSTEM, 
                                                                OTF2_PARADIGM_PTHREAD, 
                                                                OTF2_PARADIGM_HMPP, 
                                                                OTF2_PARADIGM_OMPSS, 
                                                                OTF2_PARADIGM_HARDWARE, 
                                                                OTF2_PARADIGM_GASPI, 
                                                                OTF2_PARADIGM_UPC, 
                                                                OTF2_PARADIGM_SHMEM, 
                                                                OTF2_PARADIGM_WINTHREAD, 
                                                                OTF2_PARADIGM_QTTHREAD, 
                                                                OTF2_PARADIGM_ACETHREAD, 
                                                                OTF2_PARADIGM_TBBTHREAD, 
                                                                OTF2_PARADIGM_OPENACC, 
                                                                OTF2_PARADIGM_OPENCL, 
                                                                OTF2_PARADIGM_MTAPI, 
                                                                OTF2_PARADIGM_SAMPLING, 
                                                                OTF2_PARADIGM_NONE }; // 22 -number of otf2 paradigms....



ColorPalette::ColorPalette()
{
    palette.clear();
    color_combinations.clear();
    initialize();
}
    
    
void    
ColorPalette::initialize()
{
    defineColors();
}
 
 
 
void    
ColorPalette::defineColors()
{
    double tmp_storage;
    for ( uint32_t i=0; i< NUM_OTF_PARADIGMS; ++i)
    {
        palette[ otf2_paradigms[i] ] = getColorCombination ( QColor::fromHsvF( modf( (double) ( i * 200) / (double)NUM_OTF_PARADIGMS, &tmp_storage) , 1., 1., 1.) ); 
    }
}
 
std::pair<QPen, QBrush>
ColorPalette::getColorCombination( QColor _color)
{
    QPen pen;
    pen.setColor(_color);
    QBrush brush;
    _color.setAlphaF(0.3);
    brush.setColor( _color );
    brush.setStyle( Qt::SolidPattern );
    std::pair< QPen, QBrush> _pair = std::make_pair( pen, brush );
    return _pair;
}
 
 
    
std::pair<QPen, QBrush>
ColorPalette::getColoring( const TraceEvent * _te)
{   
    std::pair< QPen, QBrush> _pair = palette[ _te->get_paradigm() ];
    if (!(trace_globals->detailed))
        return _pair;
    
    
    QColor _c = _pair.first.color();
    _c.setAlphaF((_te->is_suppressed())?0.1:0.7);
    _pair.first.setColor(_c);
     _c = _pair.second.color();
    _c.setAlphaF((_te->is_suppressed())?0.1:0.7);
    _pair.second.setColor(_c);
    
    return _pair;
}
