/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2020                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/



#include "blade-config.h"
#include "TraceEventGroupBarrierCompletionAnalyzer.h"
#include "Service.h"
#include "Trace.h"

extern Trace* trace;

TraceEventGroupBarrierCompletionAnalyzer::TraceEventGroupBarrierCompletionAnalyzer()
{

    name = "Barrier Completion";
    description = "This pattern refers to\n"
                  "the time spent in MPI \n"
                  "barriers after the first\n"
                  "process has left the\n"
                  "operation. ";
    min_max = (uint64_t)-1;
    
}

TraceEventGroupBarrierCompletionAnalyzer::~TraceEventGroupBarrierCompletionAnalyzer()
{
}


bool
TraceEventGroupBarrierCompletionAnalyzer::probe(std::vector< TraceEvent*>& events)
{
    bool only_barrier = true;
    for (std::vector< TraceEvent*>::const_iterator iter = events.begin(); iter != events.end(); ++iter)
    {
        TraceEvent * _e = *iter;
        only_barrier &= (_e->get_region_role( ) == OTF2_REGION_ROLE_BARRIER);
    }
    if (!only_barrier)
        return false;
    
    min_max = (uint64_t)-1;
    for (std::vector<TraceEvent*>::iterator iter = events.begin(); iter != events.end(); ++iter)
    {
        min_max = std::min( min_max, (*iter)->get_end_timestamp() );
    } 
    return true;
}

void
TraceEventGroupBarrierCompletionAnalyzer::analyze(std::vector<TraceEvent*>& events, int dh, int boxwidth )
{
    TraceEventGroupAnalyzer::analyze( events, dh, boxwidth  );
    clear_analyzer();
   


    int i=0;
    for (std::vector<TraceEvent*>::iterator iter = events.begin(); iter != events.end(); ++iter, ++i)
    {
        std::pair<double,double> _dp;
        _dp.first = ((double)(min_max - group_min))/(double)(group_max - group_min);
        _dp.second =  ((double)((*iter)->get_end_timestamp() - group_min))/(double)(group_max - group_min);

	TraceEventRect * _rect = new TraceEventRect((*iter)->get_end_timestamp() - min_max,  trace, (*iter), QRect(_dp.first * boxwidth, location_line_index[i]*dh, ( _dp.second - _dp.first)*boxwidth, dh) );
         
        QGraphicsSimpleTextItem * _dur_label = new QGraphicsSimpleTextItem( _rect->getDuration() );
        _dur_label->setPos(boxwidth+10,  location_line_index[i]*dh);
        dynamic_labels.push_back(_dur_label); 
        
	_rect->setPen( QColor(0, 250, 0, 200) );
        QBrush brush;
        brush.setColor( QColor(0, 250, 0, 200)  );
        brush.setStyle( Qt::SolidPattern );
        _rect->setBrush(brush );
                
	rects.push_back( _rect );
    }
}


