/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2020                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/



#include "blade-config.h"
#include "TraceEventGroupGeneralAnalyzer.h"
#include "Service.h"
#include "Trace.h"

extern Trace* trace;

TraceEventGroupGeneralAnalyzer::TraceEventGroupGeneralAnalyzer()
{

    name = "General information";
    description = "Displays various \n"
                  "parameters of the group";
    coloring = new ColorPalette();
}

TraceEventGroupGeneralAnalyzer::~TraceEventGroupGeneralAnalyzer()
{
    delete coloring;
}


bool
TraceEventGroupGeneralAnalyzer::probe(std::vector<TraceEvent*>& events)
{
    return true;
}

void
TraceEventGroupGeneralAnalyzer::analyze(std::vector<TraceEvent*>& events, int dh, int boxwidth )
{
    TraceEventGroupAnalyzer::analyze( events, dh, boxwidth  );
    clear_analyzer();
    
    std::pair<double, uint16_t> _st = trace->convert_to_seconds( group_max - group_min );
    QGraphicsSimpleTextItem * _duration_label = new QGraphicsSimpleTextItem( QString("%1%2s").arg(_st.first).arg( QString::fromStdString(bladeservices::convertToUnit (_st.second))) );
    _duration_label->setPos( boxwidth/2-30  ,  -20 );
    permanent_labels.push_back(_duration_label);

   

    int i=0;
    for (std::vector<TraceEvent*>::iterator iter = events.begin(); iter != events.end(); ++iter, ++i)
    {
        std::pair<double,double> _dp;
        _dp.first =  ((double)((*iter)->get_start_timestamp()-group_min))/(double)(group_max - group_min);
        _dp.second = ((double)((*iter)->get_end_timestamp()-group_min))/(double)(group_max - group_min);
	
	std::pair<QPen, QBrush> _colors = coloring->getColoring( (*iter) ) ;

	TraceEventRect * _rect = new TraceEventRect( trace, (*iter), QRect(_dp.first * boxwidth, location_line_index[i]*dh, ( _dp.second - _dp.first)*boxwidth, dh) );
        
        QGraphicsSimpleTextItem * _label = new QGraphicsSimpleTextItem(  QString::fromStdString( trace->get_location((*iter)->get_location()).name.c_str()) );
        _label->setPos(-100,  location_line_index[i]*dh);
        permanent_labels.push_back(_label);
        
        
         QGraphicsSimpleTextItem * _dur_label = new QGraphicsSimpleTextItem( _rect->getDuration() );
        _dur_label->setPos(boxwidth+10,  location_line_index[i]*dh);
        dynamic_labels.push_back(_dur_label); 
        
        
	_rect->setPen( _colors.first );
	_rect->setBrush( _colors.second );
	
	rects.push_back( _rect );
    }
}


