/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2020                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#include "blade-config.h"
#include <QtGui>
#include <QLabel>
#include <QWidget>
#include <QScrollArea>
#include <QGraphicsScene>
#include <QGraphicsView>
#include <QToolBar>
#include <QTimer>
#include <QThread>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QAction>
#include <QGraphicsSimpleTextItem>

#include "Trace.h"
#include "TraceEventGroupWidget.h"
#include "Service.h"
#include "TraceGlobals.h"

extern Trace * trace;
extern TraceGlobals * trace_globals;


TraceEventGroupWidget::TraceEventGroupWidget( QWidget * parent):QWidget(parent)
{
    this->setWindowTitle(tr("Details"));
    this->setWindowIcon(QIcon(":/bladeicons/icon.png"));
    QHBoxLayout * layout = new QHBoxLayout();
    QScrollArea * scroll_area = new QScrollArea();
    scene = new QGraphicsScene();
    view = new TraceGroupGraphicsView();
    view->setScene(scene);
    resize(800,500);
    setMinimumSize(800,500);
    setMaximumSize(800,500);
    
    scroll_area->setWidget( view );
    scroll_area->setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding) );
    scroll_area->setWidgetResizable(true);
    scroll_area->setVerticalScrollBarPolicy( Qt::ScrollBarAsNeeded );
    layout->addWidget(scroll_area);
    
    QVBoxLayout * descr_layout = new QVBoxLayout();
    layout->addLayout( descr_layout);
    QHBoxLayout * paging_layout = new QHBoxLayout();
    descr_layout->addLayout(paging_layout);
    paging_control = new QToolBar( );
    prev_group_button = paging_control->addAction(QIcon(":/bladeicons/left.png"), "previos group");
    next_group_button = paging_control->addAction(QIcon(":/bladeicons/right.png"), "next group");
    paging_layout->addWidget(paging_control);
    connect( prev_group_button, SIGNAL( triggered( void )), this, SLOT(setPreviousGroup()));
    connect( next_group_button, SIGNAL( triggered( void )), this, SLOT(setNextGroup()));

    
    
    
    ga = new TraceEventGroupGeneralAnalyzer();
    
    description_label = new QLabel( QString::fromStdString(ga->getDescription()));
    
    
    descr_layout->addWidget( description_label );
    
    QVBoxLayout * labels_layout = new QVBoxLayout();
    descr_layout->addLayout( labels_layout);
        
    MouseLabel* ga_label = new MouseLabel( ga,  QString::fromStdString(ga->getName()));
    analyzer_labels.push_back( ga_label );
    labels_layout->addWidget( ga_label );
    connect( ga_label, SIGNAL(mouseOver( TraceEventGroupAnalyzer* )),this, SLOT(mouseOver( TraceEventGroupAnalyzer*)));
    
    
    wait_at_barrier_a = new TraceEventGroupWaitAtBarrierAnalyzer();
    MouseLabel* wait_at_barrier_a_label = new MouseLabel(wait_at_barrier_a,   QString::fromStdString(wait_at_barrier_a->getName()));
    analyzer_labels.push_back( wait_at_barrier_a_label );
    labels_layout->addWidget( wait_at_barrier_a_label );
    connect( wait_at_barrier_a_label, SIGNAL(mouseOver( TraceEventGroupAnalyzer* )),this, SLOT(mouseOver( TraceEventGroupAnalyzer*)));
    
    barrier_completion_a = new TraceEventGroupBarrierCompletionAnalyzer();
    MouseLabel* barrier_completion_a_label = new MouseLabel( barrier_completion_a,  QString::fromStdString(barrier_completion_a->getName()));
    analyzer_labels.push_back( barrier_completion_a_label );
    labels_layout->addWidget( barrier_completion_a_label );
    connect( barrier_completion_a_label, SIGNAL(mouseOver( TraceEventGroupAnalyzer* )),this, SLOT(mouseOver( TraceEventGroupAnalyzer*)));
    
    
    wait_at_n2n_a = new TraceEventGroupWaitAtN2NAnalyzer();
    MouseLabel* wait_at_n2n_a_label = new MouseLabel(wait_at_n2n_a,   QString::fromStdString(wait_at_n2n_a->getName()));
    analyzer_labels.push_back( wait_at_n2n_a_label );
    labels_layout->addWidget( wait_at_n2n_a_label );
    connect( wait_at_n2n_a_label, SIGNAL(mouseOver( TraceEventGroupAnalyzer* )),this, SLOT(mouseOver( TraceEventGroupAnalyzer*)));
    
    n2n_completion_a = new TraceEventGroupN2NCompletionAnalyzer();
    MouseLabel* n2n_completion_a_label = new MouseLabel( n2n_completion_a,  QString::fromStdString(n2n_completion_a->getName()));
    analyzer_labels.push_back( n2n_completion_a_label );
    labels_layout->addWidget( n2n_completion_a_label );
    connect( n2n_completion_a_label, SIGNAL(mouseOver( TraceEventGroupAnalyzer* )),this, SLOT(mouseOver( TraceEventGroupAnalyzer*)));
    
    late_broadcast_a = new TraceEventGroupLateBroadcastAnalyzer();
    MouseLabel* late_broadcast_a_label = new MouseLabel( late_broadcast_a,  QString::fromStdString(late_broadcast_a->getName()));
    analyzer_labels.push_back( late_broadcast_a_label );
    labels_layout->addWidget( late_broadcast_a_label );
    connect( late_broadcast_a_label, SIGNAL(mouseOver( TraceEventGroupAnalyzer* )),this, SLOT(mouseOver( TraceEventGroupAnalyzer*)));  
     
    late_sender_a = new TraceEventGroupLateSenderAnalyzer();
    MouseLabel* late_sender_a_label = new MouseLabel( late_sender_a,  QString::fromStdString(late_sender_a->getName()));
    analyzer_labels.push_back( late_sender_a_label );
    labels_layout->addWidget( late_sender_a_label );
    connect( late_sender_a_label, SIGNAL(mouseOver( TraceEventGroupAnalyzer* )),this, SLOT(mouseOver( TraceEventGroupAnalyzer*)));  
    
    late_receiver_a = new TraceEventGroupLateReceiverAnalyzer();
    MouseLabel* late_receiver_a_label = new MouseLabel( late_receiver_a,  QString::fromStdString(late_receiver_a->getName()));
    analyzer_labels.push_back( late_receiver_a_label );
    labels_layout->addWidget( late_receiver_a_label );
    connect( late_receiver_a_label, SIGNAL(mouseOver( TraceEventGroupAnalyzer* )),this, SLOT(mouseOver( TraceEventGroupAnalyzer*)));  
    
    early_reduce_a = new TraceEventGroupEarlyReduceAnalyzer();
    MouseLabel* early_reduce_a_label = new MouseLabel( early_reduce_a,  QString::fromStdString(early_reduce_a->getName()));
    analyzer_labels.push_back( early_reduce_a_label );
    labels_layout->addWidget( early_reduce_a_label );
    connect( early_reduce_a_label, SIGNAL(mouseOver( TraceEventGroupAnalyzer* )),this, SLOT(mouseOver( TraceEventGroupAnalyzer*)));  
    
    
    setLayout(layout);
    hide();
}

TraceEventGroupWidget::~TraceEventGroupWidget() 
{
    delete ga;
    delete wait_at_barrier_a;
    delete barrier_completion_a;
    delete wait_at_n2n_a;
    delete n2n_completion_a;
    delete early_reduce_a;
    delete late_broadcast_a;
    delete late_sender_a;
    delete late_receiver_a;
};



void
TraceEventGroupWidget::mouseOver(TraceEventGroupAnalyzer* _an)
{
    for (std::vector< QGraphicsSimpleTextItem* >::iterator iter = dynamic_labels.begin(); iter != dynamic_labels.end(); ++iter)
    {
            scene->removeItem(  (*iter) );
    }
    for (std::vector< TraceEventRect* >::iterator iter = dynamic_rects.begin(); iter != dynamic_rects.end(); ++iter)
    {
            scene->removeItem(  (*iter) );
    }   

    
    
    
    dynamic_labels.clear();
    dynamic_rects.clear();
    description_label->setText(QString::fromStdString(_an->getDescription()));
    std::vector< QGraphicsSimpleTextItem * >& _labels = _an->get_dynamic_labels();
    for (std::vector< QGraphicsSimpleTextItem* >::iterator iter = _labels.begin(); iter != _labels.end(); ++iter)
    {
        scene->addItem(  (*iter) );
        dynamic_labels.push_back( *iter);
    }
    if ( _an !=  ga)
    {
        std::vector< TraceEventRect* >& _dyn_rects = _an->get_rects();
        for (std::vector< TraceEventRect* >::iterator iter = _dyn_rects.begin(); iter != _dyn_rects.end(); ++iter)
        {
            scene->addItem(  (*iter) );
            dynamic_rects.push_back( *iter);
        }
    }

    
}


void
TraceEventGroupWidget::displayEvents(std::vector<TraceEvent*>& _events, TraceEventGroup * _group)
{
    events = _events;
    trace_globals->detailed = true;
    
    for (std::vector<TraceEvent*>::iterator iter= events.begin(); iter != events.end(); ++iter)
    {
        (*iter)->set_suppressed(false);
    }
    
    current_group =_group;
    paging_control->setEnabled(current_group != NULL);
    if (current_group != NULL)
    {
        prev_group_button->setEnabled( current_group->get_prev_group() != NULL);
        next_group_button->setEnabled( current_group->get_next_group() != NULL);
    }
    
    
    scene->clear();
    dynamic_labels.clear();
    dynamic_rects.clear();
    int dh = 30; // heigh of a location line
    int dw = 300; // width of the box;
    
    scene->addRect( 0, 0, dw, dh*events.size(), QPen(QColor(200,200,200,127)));

    if (ga->probe(events ))
    {
        ga->analyze(  events, dh, dw );
        
        double _dw= (ga->get_group_max() - ga->get_group_min())/(20.);
        uint64_t _min = (dw> ga->get_group_min())?0.:(ga->get_group_min() - (uint64_t)dw);
        uint64_t _max = (dw+ ga->get_group_max() > trace->get_end_timestamp())?trace->get_end_timestamp():(ga->get_group_max() + (uint64_t)dw);
        
        z_filter->set_zoom_window( _min, _max);
        
        
        
        std::vector< TraceEventRect* >& _rects = ga->get_rects();
        for (std::vector< TraceEventRect* >::iterator iter = _rects.begin(); iter != _rects.end(); ++iter)
        {
            scene->addItem(  (*iter) );
        }

        std::vector< QGraphicsSimpleTextItem * >& _labels = ga->get_permanent_labels();
        for (std::vector< QGraphicsSimpleTextItem* >::iterator iter = _labels.begin(); iter != _labels.end(); ++iter)
        {
            scene->addItem(  (*iter) );
        }
    }
    
    
    analyzer_labels[1]->hide();
    if (wait_at_barrier_a->probe(events ))
    {
        analyzer_labels[1]->show();
        wait_at_barrier_a->analyze(  events, dh, dw );
        

        std::vector< QGraphicsSimpleTextItem * >& _wab_labels = wait_at_barrier_a->get_permanent_labels();
        for (std::vector< QGraphicsSimpleTextItem* >::iterator iter = _wab_labels.begin(); iter != _wab_labels.end(); ++iter)
        {
            scene->addItem(  (*iter) );
        }
    }
    analyzer_labels[2]->hide();
    if (barrier_completion_a->probe(events ))
    {
        analyzer_labels[2]->show();
        barrier_completion_a->analyze(  events, dh, dw );

        std::vector< QGraphicsSimpleTextItem * >& _bc_labels = barrier_completion_a->get_permanent_labels();
        for (std::vector< QGraphicsSimpleTextItem* >::iterator iter = _bc_labels.begin(); iter != _bc_labels.end(); ++iter)
        {
            scene->addItem(  (*iter) );
        }
    }  
    
        
    analyzer_labels[3]->hide();
    if (wait_at_n2n_a->probe(events ))
    {
        analyzer_labels[3]->show();
        wait_at_n2n_a->analyze(  events, dh, dw );
        

        std::vector< QGraphicsSimpleTextItem * >& _wab_labels = wait_at_n2n_a->get_permanent_labels();
        for (std::vector< QGraphicsSimpleTextItem* >::iterator iter = _wab_labels.begin(); iter != _wab_labels.end(); ++iter)
        {
            scene->addItem(  (*iter) );
        }
    }
    analyzer_labels[4]->hide();
    if (n2n_completion_a->probe(events ))
    {
        analyzer_labels[4]->show();
        n2n_completion_a->analyze(  events, dh, dw );

        std::vector< QGraphicsSimpleTextItem * >& _bc_labels = n2n_completion_a->get_permanent_labels();
        for (std::vector< QGraphicsSimpleTextItem* >::iterator iter = _bc_labels.begin(); iter != _bc_labels.end(); ++iter)
        {
            scene->addItem(  (*iter) );
        }
    }  
    
    analyzer_labels[5]->hide();
    if (late_broadcast_a->probe(events ))
    {
        analyzer_labels[5]->show();
        late_broadcast_a->analyze(  events, dh, dw );

        std::vector< QGraphicsSimpleTextItem * >& _bc_labels = late_broadcast_a->get_permanent_labels();
        for (std::vector< QGraphicsSimpleTextItem* >::iterator iter = _bc_labels.begin(); iter != _bc_labels.end(); ++iter)
        {
            scene->addItem(  (*iter) );
        }
    }     
    
    analyzer_labels[6]->hide();
    if (late_sender_a->probe(events ))
    {
        analyzer_labels[6]->show();
        late_sender_a->analyze(  events, dh, dw );

        std::vector< QGraphicsSimpleTextItem * >& _bc_labels = late_sender_a->get_permanent_labels();
        for (std::vector< QGraphicsSimpleTextItem* >::iterator iter = _bc_labels.begin(); iter != _bc_labels.end(); ++iter)
        {
            scene->addItem(  (*iter) );
        }
    }    
     
    analyzer_labels[7]->hide();
    if (late_receiver_a->probe(events ))
    {
        analyzer_labels[7]->show();
        late_receiver_a->analyze(  events, dh, dw );

        std::vector< QGraphicsSimpleTextItem * >& _bc_labels = late_receiver_a->get_permanent_labels();
        for (std::vector< QGraphicsSimpleTextItem* >::iterator iter = _bc_labels.begin(); iter != _bc_labels.end(); ++iter)
        {
            scene->addItem(  (*iter) );
        }
    }   
 
      
    analyzer_labels[8]->hide();
    if (early_reduce_a->probe(events ))
    {
        analyzer_labels[8]->show();
        early_reduce_a->analyze(  events, dh, dw );

        std::vector< QGraphicsSimpleTextItem * >& _bc_labels = early_reduce_a->get_permanent_labels();
        for (std::vector< QGraphicsSimpleTextItem* >::iterator iter = _bc_labels.begin(); iter != _bc_labels.end(); ++iter)
        {
            scene->addItem(  (*iter) );
        }
    }   
 
    show();
}



void
TraceEventGroupWidget::setNextGroup()
{
        
    for (std::vector<TraceEvent*>::iterator iter= events.begin(); iter != events.end(); ++iter)
        (*iter)->set_suppressed(true);
    
    if (current_group== NULL)
        return;
    TraceEventGroup * _next_group = current_group->get_next_group();

    if (_next_group == NULL)
        return;
    
    std::vector<TraceEvent*> _events;
    _events= _next_group->get_events();
    std::sort(_events.begin(), _events.end(), bladeservices::compare_locations<TraceEvent*>);
    
    displayEvents( _events, _next_group );
    
    
}


void
TraceEventGroupWidget::setPreviousGroup()
{
            
    for (std::vector<TraceEvent*>::iterator iter= events.begin(); iter != events.end(); ++iter)
        (*iter)->set_suppressed(true);
        
    if (current_group== NULL)
        return;
    TraceEventGroup * _prev_group = current_group->get_prev_group();
    if (_prev_group == NULL)
        return;
    
    std::vector<TraceEvent*> _events;
    _events= _prev_group->get_events();
    std::sort(_events.begin(), _events.end(), bladeservices::compare_locations<TraceEvent*>);
    
    displayEvents( _events, _prev_group );
    
    
}
    
    
void
TraceEventGroupWidget::closeEvent(QCloseEvent * event )
{
    trace_globals->detailed = false;
    for (std::vector<TraceEvent*>::iterator iter= events.begin(); iter != events.end(); ++iter)
        (*iter)->set_suppressed(true);
     z_filter->no_zoom();
}

void
TraceEventGroupWidget::hideEvent(QHideEvent * event )
{
    trace_globals->detailed = false;
    for (std::vector<TraceEvent*>::iterator iter= events.begin(); iter != events.end(); ++iter)
        (*iter)->set_suppressed(true);
     z_filter->no_zoom();    
}
