/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2020                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#include "blade-config.h"
#include <iostream>

#include "TraceLoaderThread.h"
#include "TraceLoadEnterFilter.h"
#include "TraceLoadLeaveFilter.h"

TraceLoaderThread::TraceLoaderThread(QString _path, cube::Cube* _cube):QThread(),path(_path),cube(_cube)
{
    trace_load_enter_filter = NULL;
    trace_load_leave_filter = NULL;
    trace_loader = new TraceLoader( path.toStdString(),cube );
    trace = trace_loader->getTrace();
}
    
    
TraceLoaderThread::~TraceLoaderThread()
{
    delete trace_loader;
}

   
   
void
TraceLoaderThread::run()
{
    trace_loader->loadEvents(trace_load_enter_filter, trace_load_leave_filter, grouping);    
    emit traceLoaded();
}
   

      
double
TraceLoaderThread::get_progress()
{
    return trace_loader->get_progress();
}
 
       
std::string&
TraceLoaderThread::get_progress_message()
{
    return trace_loader->get_progress_message();
}
 
 
void
TraceLoaderThread::set_load_filters( TraceLoadEnterFilter * _tlf_enter, TraceLoadLeaveFilter * _tlf_leave, bool _grouping) 
{
    trace_load_enter_filter = _tlf_enter;
    trace_load_leave_filter = _tlf_leave;
    grouping = _grouping;
}
 
