/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2020                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/

#include "jengafett-config.h"
#include <iostream>
#include <fstream>
#include <limits>

#include "CubeDataExtract.h"


using namespace jengafett_plugin::datalib;
 
CubeDataExtract::CubeDataExtract(cube::CubeProxy* _cube):cube(_cube)
{
}


CubeDataExtract::~CubeDataExtract()
{
}
    
    
    
             

void
CubeDataExtract::prepare_arguments( const cube::list_of_sysresources& _sysres)
{
    arguments.clear();
    
    for(cube::list_of_sysresources::const_iterator iter = _sysres.begin(); iter != _sysres.end(); ++iter     )
    {
        cube::Sysres* _working_sysres = NULL;
        cube::SystemTreeNode* _working_stn = NULL;
        
        if ( iter->first->isLocation() )
        {
            _working_sysres = iter->first->get_parent();
        }
        else
        {
            _working_sysres = const_cast<cube::Sysres*>(iter->first);
        }
        
        _working_stn =dynamic_cast<cube::SystemTreeNode*>(const_cast<cube::Sysres*>(iter->first));
        
        if ( _working_sysres->isSystemTreeNode())
        {
            for (uint64_t i=0; i< _working_stn->num_groups(); ++i )
            {
                arguments.push_back( _working_stn->get_location_group(i));
            }
        }
        for (uint64_t i=0; i< _working_sysres->num_children(); ++i )
        {
            arguments.push_back( _working_sysres->get_child(i));
        }
    }
    
    
}
    
    
void
CubeDataExtract::getArguments(const cube::list_of_sysresources& _sysres, std::vector<double>& _vec, QMap< double, QString > & _vec_labeled, double& maxx)
{
    prepare_arguments( _sysres );
    
    maxx = -std::numeric_limits<double>::max();
    uint64_t i = 0;
    for (std::vector<cube::Sysres*>::const_iterator iter = arguments.begin(); iter != arguments.end(); ++iter, ++i)
    {
        double x = i; //(*iter)->get_id();
        maxx = std::max( x, maxx ); 
        _vec_labeled[ x ] = QString::fromStdString( (*iter)->get_name() );
        _vec.push_back( x );
    }
}


void
CubeDataExtract::getData(   const cube::Metric* _metric, 
                            const cube::list_of_cnodes& _cnodes,
                            std::vector<double>& _vec, 
                            std::vector<double>& _max, 
                            double& maxy) const
{
 
    if (_metric == NULL) // dummy call, to align groupd properly
    {
         for ( size_t i=0; i<arguments.size(); ++i)
        {
            _vec.push_back( 0. );
            _max.push_back( 0. );
            
        }
        return;
    }
    
    cube::list_of_metrics _metrics;
    _metrics.push_back(
            std::make_pair<cube::Metric*, cube::CalculationFlavour>( const_cast<cube::Metric*>(_metric) , cube::CUBE_CALCULATE_INCLUSIVE)
        );
    
    cube::value_container       inclusive_values;
    cube::value_container       exclusive_values;
    cube->getSystemTreeValues(_metrics,
                         _cnodes,
                         inclusive_values,
                         exclusive_values );
    
    
    maxy = -std::numeric_limits<double>::max();
    bool create_new_own_max= (_max.size()==0);
    for ( size_t i=0; i<arguments.size(); ++i)
    {
        
            
            double _value = (inclusive_values[ arguments[i]->get_sys_id()] != NULL)? inclusive_values[ arguments[i]->get_sys_id()]->getDouble():0.;
            delete inclusive_values[ arguments[i]->get_sys_id()];
            delete exclusive_values[ arguments[i]->get_sys_id()];
            
/*            
            cube-> get_sev( 
                                _metrics,
                                _cnodes,
                                arguments[i],
                                cube::CUBE_CALCULATE_INCLUSIVE
                                        );*/
        
        _vec.push_back( _value );
        if (create_new_own_max)
        {
            _max.push_back( _value );
        }
        else
        {
            _max[i] = std::max( _value, _max[i] ); 
        }
        maxy = std::max( _value, maxy ); 
    }
    
}
