/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2019                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef JENGAFETT_PLUGIN_WIDGET_H
#define JENGAFETT_PLUGIN_WIDGET_H
#include <QtGui>
#include <QLabel>
#include <QWidget> 
#include <QMultiMap>
#include <qcustomplot.h>
#include "CubeProxy.h"
#include "PluginServices.h"
#include "TabInterface.h"
#include "CubeDataExtract.h"
#include "JengaFettToolBar.h"


namespace jengafett_plugin
{
    
    namespace gui 
    {
    
        enum DisplayStates { JENGAFETT_ABSOLUTE=0, 
                             JENGAFETT_COMMON_MAX=1,
                             JENGAFETT_EVERYBAR_MAX=2};
                            
        class JengaFettPluginWidget : public QWidget
        {
            Q_OBJECT
            
        private:
            cube::CubeProxy*    cube;
            JengaFettToolBar * toolBar;
            QCustomPlot * customPlot;
            QSharedPointer<QCPAxisTickerText> textTicker;
            QCPBars *    bars;
            QCPBarsGroup *prime_group, *sec_group;
            
            jengafett_plugin::datalib::CubeDataExtract * data;
            QVector<double> _x_to_plot;
            double maxx;
            uint16_t num_bar_groups;
            std::map<const cube::Metric*, double> maxes_y;
            bool    logPlot;
            
            DisplayStates displayState;
    
            QMultiMap<cube::Metric* , cube::Metric* > metric_groups; // using root of metric as a key, and metric as a value
            QMultiMap<cube::Metric* , cube::CalculationFlavour > metric_state;
            
            double
            addBarsToPlot( const  cube::Metric*, const  cube::list_of_cnodes&, const int, const int);
            
            uint16_t 
            calculateGroupsNumber(const QList<cubepluginapi::TreeItem*>& ) const;
            
        private slots:

            
        public slots:
          
            
        public:
            JengaFettPluginWidget(cube::CubeProxy*, JengaFettToolBar* , QWidget * parent = NULL);
            ~JengaFettPluginWidget();
            
            void
            selectedItems( const QList<cubepluginapi::TreeItem*>&, 
                           const QList<cubepluginapi::TreeItem*>&, 
                           const QList<cubepluginapi::TreeItem*>&);
            
            void
            addMetricToGraph( cube::Vertex * );
            
            void
            clearJengaPlot( );
            
            
            void
            setAbsolute ( );

            void
            setCommonMax ( );

            void
            setEveryBarMax( );
            
            void
            setLogPlot( bool );
        };
    };
};
#endif // TESTPLUGIN1_H
