/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2022                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#include "config.h"
#include <iostream>
#include <QVBoxLayout>
#include "ScorePionFilterSetWidget.h"
#include "ScorePionPluginWidget.h"
#include "ExcludeFileFilterRulesSet.h"
#include "ExcludeRegionFilterRulesSet.h"
#include "IncludeFileFilterRulesSet.h"
#include "IncludeRegionFilterRulesSet.h"


#if QT_VERSION <= QT_VERSION_CHECK( 5, 14, 0 )
#define SKIP QString::SkipEmptyParts
#else
#define SKIP Qt::SkipEmptyParts
#endif

using namespace scorepion_plugin::gui;
using namespace scorepion_plugin::filters;

ScorePionFilterSetWidget::ScorePionFilterSetWidget(  scorepion_plugin::filters::FilterType _rules_type,  QWidget * parent ) : QWidget( parent ),type(_rules_type)
{

    switch ( type)
    {
        case scorepion_plugin::filters::INCLUDE_FILE:
            title = tr("Include File");
            rules = new scorepion_plugin::filters::IncludeFileFilterRulesSet();
            break;
        case scorepion_plugin::filters::INCLUDE_REGION:
            title = tr("Include Region");
            rules = new scorepion_plugin::filters::IncludeRegionFilterRulesSet();
            break;
        case scorepion_plugin::filters::EXCLUDE_FILE:
            title = tr("Exclude File");
            rules = new scorepion_plugin::filters::ExcludeFileFilterRulesSet();
            break;
        case scorepion_plugin::filters::EXCLUDE_REGION:
        default:
            title = tr("Exclude Region");
            rules = new scorepion_plugin::filters::ExcludeRegionFilterRulesSet();
            break;
    }

    QVBoxLayout* layout = new QVBoxLayout();
    this->setLayout( layout );
    tab = new QTabWidget();
    layout->addWidget(tab);
    tab->setTabsClosable( true );

    content = new QTextEdit();
    tab->addTab( content, title);
    connect( tab, SIGNAL( tabCloseRequested(int)), this, SLOT(removeSelf(int)));
    connect( content, SIGNAL(textChanged()), this, SLOT(changedFilterSlot()) );
}



ScorePionFilterSetWidget::ScorePionFilterSetWidget( scorepion_plugin::filters::FilterRulesSet * filter_rules_set,  QWidget * parent ) : ScorePionFilterSetWidget( filter_rules_set->getFilterType(), parent)
{
    delete rules;
    rules = filter_rules_set;
    rulesChanged();
}


void
ScorePionFilterSetWidget::rulesChanged()
{
    std::stringstream sstr;
    rules->print( sstr );
    QString str = QString::fromStdString( sstr.str() );

    content->setText( str );
}

void
ScorePionFilterSetWidget::removeSelf( int index )
{
    (void) index;
    static_cast<ScorePionPluginWidget*>(parentWidget()->parentWidget())->removeFilterSetWidget( this );
}


void
ScorePionFilterSetWidget::changedFilterSlot()
{
    rules->clearRules();
    QString rules_content = content->document()->toPlainText();
    QStringList rules_strings = rules_content.split("\n", SKIP );
    QStringList::const_iterator constIterator;
    for (constIterator = rules_strings.constBegin(); constIterator != rules_strings.constEnd(); ++constIterator)
    {
        std::string _rule =(*constIterator).trimmed().toStdString();
        rules->AddRule( _rule );
    }
    emit changedFilter();
}
