/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2018                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#include "FilterRulesSet.h" 

namespace scorepion_plugin
{
    namespace filters
    {

        FilterRulesSet::~FilterRulesSet()
        {
            clearRules();
        }

        
        bool
        FilterRulesSet::isEmpty() const 
        {
            return ((rules.size() == 0));
        }
        
        
        void
        FilterRulesSet::clearRules( ) 
        {
            for (std::vector<FilterRule*>::iterator iter = rules.begin(); iter != rules.end(); ++iter)
            {
                delete *iter;
            }
            rules.clear( );
        }
             
        void
        FilterRulesSet::AddRule( FilterRule * _rule) 
        {
            rules.push_back( _rule);
        }
        
        
        FilterResult
        FilterRulesSet::apply( cube::Vertex * _v) const
        {
            FilterResult result = NO_MATCH;
            for (std::vector<FilterRule*>::const_iterator iter = rules.begin(); iter != rules.end(); ++iter)
            {
                FilterResult _result =  (*iter)->apply( _v );
                result = ( _result != NO_MATCH )? _result:result;
            }
            return result;
        }
               
        
        void
        FilterRulesSet::print(std::ostream& out, bool intel) const
        {
            if (!isEmpty())
            {
                for (std::vector<FilterRule*>::const_iterator iter = rules.begin(); iter != rules.end(); ++iter)
                {
                    (*iter)->print(out, intel);
                    if ( intel )
                    {
                    	if (rules_type == INCLUDE_FILE || rules_type == EXCLUDE_FILE)
                    	{
                    		out << ":.*";
                    	}
            		out << " ";
            	        out <<   ( (rules_type==INCLUDE_FILE || rules_type == INCLUDE_REGION)?"ON":"OFF"); 
                 	out << std::endl;
                    }
                    out << std::endl;
                }
            }
         }
          
  
        void
        FilterRulesSet::writeIntel(std::ostream& out) const
        {
            print( out, true );
        }
  
  
        void
        FilterRulesSet::write(std::ostream& out) const
        {
        
            switch ( rules_type )
            {
                case INCLUDE_FILE:
                    out << "SCOREP_FILE_NAMES_BEGIN" << std::endl << "INCLUDE" << std::endl;
                    break;
                case INCLUDE_REGION:
                    out << "SCOREP_REGION_NAMES_BEGIN" << std::endl << "INCLUDE" << std::endl;
                    break;
                case EXCLUDE_FILE:
                    out << "SCOREP_FILE_NAMES_BEGIN" << std::endl << "EXCLUDE" << std::endl;
                    break;
                case EXCLUDE_REGION:
                default:
                    out << "SCOREP_REGION_NAMES_BEGIN" << std::endl << "EXCLUDE" << std::endl;
                    break;
            }
            
            
            print( out );
        
            switch ( rules_type )
            {
                case INCLUDE_FILE:
                    out << "SCOREP_FILE_NAMES_END" << std::endl;
                    break;
                case INCLUDE_REGION:
                    out << "SCOREP_REGION_NAMES_END" << std::endl;
                    break;
                case EXCLUDE_FILE:
                    out << "SCOREP_FILE_NAMES_END" << std::endl;
                    break;
                case EXCLUDE_REGION:
                default:
                    out << "SCOREP_REGION_NAMES_END" << std::endl;
                    break;
            }
          
        }
  
  
            
        std::ostream& 
        FilterRulesSet::operator<< (std::ostream& out)
        {
            print( out );
            return out;
        }
        

        
    };
 };

